########################
# Lecture 9
# Panel Data
# POLS 509: The Linear Model
# Dr. Justin Esarey
########################

install.packages(c("pcse","plm","Hmisc","lme4","arm"), dep=T)

library(foreign)
library(pcse)	# a package enabling the estimation of Panel-Corrected Standard Errors
library(plm)	# the panel linear models library.
library(Hmisc)


# Check out the documentation for the PLM package:
# http://cran.r-project.org/web/packages/plm/plm.pdf
# http://cran.r-project.org/web/packages/plm/vignettes/plm.pdf



read.dta("lecture9.dta")->data    

# Data prep
############

data<-subset(data, data$year>1977 & data$year<2001)
data<-subset(data, data$stname!="District of Columbia")
attach(data)

# per capita police spending, state + local
policePC = policeex1/totalpop
# per capita welfare spending, state + local
welfarePC = welfare1/totalpop
# per capita education spending, state + local
edspendPC = edspend1/totalpop
# log population
logpop = log(totalpop)
# log per capita income
logpcinc = log(PInccapC)


# Consider first a basic regression with no accounting for unit heterogeneity. 
model<-lm(murder~adperI+policePC+edspendPC+logpcinc+logpop)
summary(model)

# We could improve this basic model by correcting for spatial and temporal correlation using PSCEs.
# Annoyingly, this doesn't handle NAs very well. We must do so manually.
data.na<-cbind(stfips, year, murder, adperI, policePC, edspendPC, logpcinc, logpop)
data.na<-as.data.frame(na.omit(data.na))
model.na<-lm(murder~adperI+policePC+edspendPC+logpcinc+logpop, data=data.na)
model.pcse<-pcse(model.na, groupN=data.na$stfips, groupT=data.na$year)
summary(model.pcse)




# Next, consider the fixed effects model (with varying intercepts). 
model.fe<-lm(murder~adperI+policePC+edspendPC+logpcinc+logpop+factor(stfips)-1)
summary(model.fe)
coef(summary(model.fe))[1:5,]	# display non-dummy variables only

# can also do this with the plm package
data.plm<-pdata.frame(data, index=c("stfips", "year"), drop.index=T, row.names=T)
model.fe.plm<-plm(murder~adperI+policePC+edspendPC+logpcinc+logpop, data=data.plm, model="within")	# this is the fixed effects model
summary(model.fe.plm)






# Finally, consider the random effects model. 
model.re.plm<-plm(murder~adperI+policePC+edspendPC+logpcinc+logpop, data=data.plm, model="random")
summary(model.re.plm)


# We can do a Hausman test:
phtest(model.fe.plm, model.re.plm)



# The plm package handles lagged DVs quite nicely. 
model.lag<-plm(murder~lag(murder,1)+adperI+policePC+edspendPC+logpcinc+logpop, data=data.plm, model="pooling")
summary(model.lag)




